# coding: utf-8

"""
    LMK API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr, validator

class JupyterSessionState(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    type: StrictStr = ...
    url: Optional[StrictStr] = None
    notebook_name: Optional[StrictStr] = Field(None, alias="notebookName")
    shell_state: Optional[StrictStr] = Field(None, alias="shellState")
    cell_state: Optional[StrictStr] = Field(None, alias="cellState")
    cell_text: Optional[StrictStr] = Field(None, alias="cellText")
    cell_error: Optional[StrictStr] = Field(None, alias="cellError")
    execution_num: Optional[float] = Field(None, alias="executionNum")
    cell_started_at: Optional[StrictStr] = Field(None, alias="cellStartedAt")
    cell_finished_at: Optional[StrictStr] = Field(None, alias="cellFinishedAt")
    notify_on: StrictStr = Field(..., alias="notifyOn")
    notify_channel: Optional[StrictStr] = Field(None, alias="notifyChannel")
    __properties = ["type", "url", "notebookName", "shellState", "cellState", "cellText", "cellError", "executionNum", "cellStartedAt", "cellFinishedAt", "notifyOn", "notifyChannel"]

    @validator('type')
    def type_validate_enum(cls, v):
        if v not in ('jupyter'):
            raise ValueError("must validate the enum values ('jupyter')")
        return v

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> JupyterSessionState:
        """Create an instance of JupyterSessionState from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> JupyterSessionState:
        """Create an instance of JupyterSessionState from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return JupyterSessionState.parse_obj(obj)

        _obj = JupyterSessionState.parse_obj({
            "type": obj.get("type"),
            "url": obj.get("url"),
            "notebook_name": obj.get("notebookName"),
            "shell_state": obj.get("shellState"),
            "cell_state": obj.get("cellState"),
            "cell_text": obj.get("cellText"),
            "cell_error": obj.get("cellError"),
            "execution_num": obj.get("executionNum"),
            "cell_started_at": obj.get("cellStartedAt"),
            "cell_finished_at": obj.get("cellFinishedAt"),
            "notify_on": obj.get("notifyOn"),
            "notify_channel": obj.get("notifyChannel")
        })
        return _obj

